#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2016 Earth Sciences Department, BSC-CNS
#

"""nc2geojson module.

This module provide conversion from netCDF to geojson format.

"""

import netCDF4 as nc
import numpy as np
import json
import jsonator
import os
import os.path
import sys

np.set_printoptions(precision=2)

CURRENT_PATH = os.path.abspath(os.path.dirname(__file__))

# VARS = json.load(open('conf/vars.json'))

# netcdf forecast_yearly_anomalies_BSC_Nov2018_tas {
# dimensions:
#         longitude = 320 ;
#         latitude = 160 ;
# variables:
#         double longitude(longitude) ;
#                 longitude:units = "degrees_east" ;
#                 longitude:long_name = "longitude" ;
#         double latitude(latitude) ;
#                 latitude:units = "degrees_north" ;
#                 latitude:long_name = "latitude" ;
#         float ano_s2018_f2019(latitude, longitude) ;
#                 ano_s2018_f2019:units = "degrees" ;
#                 ano_s2018_f2019:_FillValue = -1.f ;
#                 ano_s2018_f2019:long_name = "tas  anomaly first year mean of
#                 forecast initialised in November 2018" ;
#         float ano_s2018_f2020(latitude, longitude) ;
#                 ano_s2018_f2020:units = "degrees" ;
#                 ano_s2018_f2020:_FillValue = -1.f ;
#                 ano_s2018_f2020:long_name = "tas  anomaly second year mean of
#                 forecast initialised in November 2018" ;
#         float ano_s2018_f2021(latitude, longitude) ;
#                 ano_s2018_f2021:units = "degrees" ;
#                 ano_s2018_f2021:_FillValue = -1.f ;
#                 ano_s2018_f2021:long_name = "tas  anomaly third year mean of
#                 forecast initialised in November 2018" ;
#         float ano_s2018_f2022(latitude, longitude) ;
#                 ano_s2018_f2022:units = "degrees" ;
#                 ano_s2018_f2022:_FillValue = -1.f ;
#                 ano_s2018_f2022:long_name = "tas  anomaly fourth year mean of
#                 forecast initialised in November 2018" ;
#         float ano_s2018_f2023(latitude, longitude) ;
#                 ano_s2018_f2023:units = "degrees" ;
#                 ano_s2018_f2023:_FillValue = -1.f ;
#                 ano_s2018_f2023:long_name = "tas  anomaly fifth year mean of
#                 forecast initialised in November 2018" ;
#         float ano_s2018_f2018_2023(latitude, longitude) ;
#                 ano_s2018_f2018_2023:units = "degrees" ;
#                 ano_s2018_f2018_2023:_FillValue = -1.f ;
#                 ano_s2018_f2018_2023:long_name = "tas  anomaly fifth year
#                 mean of forecast initialised in November 2018" ;
# }

VARS = [
    'ano_s2018_f2019',
    'ano_s2018_f2020',
    'ano_s2018_f2021',
    'ano_s2018_f2022',
    'ano_s2018_f2023',
    'ano_s2018_f2018_2023',
]


def nc2geojson(outdir='.', filelist=[], outfile_tpl=''):
    """ NetCDF(s) to geojson converter """

    print("Converting netCDF(s) ...")
    outfiles = []

    if isinstance(filelist, str):
        # file is global
        filelist = [filelist]

    # loop over files
    for filename in filelist:
        print("current file %s", filename)
        fp = nc.Dataset(filename)
#         tim = fp.variables['time']
#         timevals = tim[:].copy()
#         _, _, date, _ = tim.units.split()[:4]
        lats = np.round(fp.variables['latitude'][:], decimals=2)
        lons = np.round(fp.variables['longitude'][:], decimals=2)

        for variable in VARS:

            # mul = VARS[variable]['mul']
            levels = np.arange(-6, 7)  # VARS[variable]['bounds']

            # loop over timesteps

            features = []

            values = fp.variables[variable][:]

            geojson = jsonator.contourf(lons, lats, values,
                                        levels=levels,
                                        cmap=None)

            if geojson:
                features.append(geojson)

            if features:
                merged = json.loads(features[0]).copy()
                for feat in features[1:]:
                    merged['features'] += json.loads(feat)['features']

                outfile = '{:02d}_{}_{}.geojson'.format(0, '2018',
                                                        variable)
                outfiles.append(outfile)
                with open(os.path.join(outdir, outfile), 'w') as out:
                    out.write(json.dumps(merged, separators=(',', ':')))

        fp.close()


if __name__ == "__main__":
    outdir = sys.argv[1]
    filenames = sys.argv[2:]
    print('outdir', outdir)
    print('fnames', filenames)
    nc2geojson(outdir, filenames)
